clear all;
close all;
clc;

% Three dimensional case

Xd = 256; Yd = 256; Zd = 26;
Sx = 2; Sy = 2; Sz = 1.0;

[X, Y, Z] = meshgrid(1:Xd, 1:Yd, 1:Zd);

N_beads = 50; % number of beads

N_X = rand(N_beads, 1)*Xd;
N_Y = rand(N_beads, 1)*Yd;
N_Z = rand(N_beads, 1)*Zd;

EXP_FUN = @(P) exp(-(((X - P(1))/Sx).^2 + ((Y - P(2))/Sy).^2 + ((Z - P(3))/Sz).^2));

S = zeros(Xd, Yd, Zd);

for i=1:N_beads,
    S = S + 500*EXP_FUN([N_X(i), N_Y(i), N_Z(i)]);
end

figure;
imshow(S(:, :, 1));
for i=1:Zd,
    image(S(:, :, i));drawnow; pause(.1);
    M(i) = getframe(gca);
end
movie2avi(M, 'Original.avi');

% Create a BW image

S_BW = (S > 100)*1;
figure;
imshow(S(:, :, 1));
for i=1:Zd,
    image(500*S_BW(:, :, i));drawnow; pause(.1);
    M(i) = getframe(gca);
end
movie2avi(M, 'Threshold.avi');

[L, N_L] = bwlabeln(S_BW, 6);

% Create the basis
dx = .1;
[X_C, Y_C, Z_C] = meshgrid(1:dx:Xd, 1:dx:Yd, 1:dx:Zd);
X_center = (Xd + 1)/2; Y_center = (Yd + 1)/2; Z_center = (Zd + 1)/2;

% This is the main matrix from which we will cut out the basis
B_MAT = exp(-(((X_C - X_center)/Sx).^2 + ((Y_C - Y_center)/Sy).^2 + ((Z_C - Z_center)/Sz).^2));

[s1, s2, s3] = size(S_BW);
for k=1:N_L,
    
    % Now find the boundaries
    Id_th = find(handles.L == k); %% find all the points that are larger than th;
    
    Id_prime = Id_th-1;
    Z_th = floor(Id_prime/(s1*s2))+1;
    Y_th = floor(mod(Id_prime, s1*s2)/s1)+1;
    X_th = mod(Id_prime, s1) +1;

    X_min = min(X_th) - 1;X_max = max(X_th) + 1;
    if X_min < 1, X_min = 1; end
    if X_max > s1, X_max = s1; end
    
    Y_min = min(Y_th) - 1;Y_max = max(Y_th) + 1;
    if Y_min < 1, Y_min = 1; end
    if Y_max > s2, Y_max = s2; end
    
    Z_min = min(Z_th) - 1;Z_max = max(Z_th) + 1;
    if Z_min < 1, Z_min = 1; end
    if Z_max > s3, Z_max = s3; end
    
    

end






