function DETECT_PARTICLES(folder_name, series_name)
% OLEG POLYAKOV | PRINCETON UNIVERSITY | 2008-2013
%clc;close all;

% SAMPLE FOLDER NAME AND SERIES NAME
%folder_name = '20110427';
%series_name = 'SpiderGFP-RedBeads(5)';
%Case_name = strcat(folder_name, '-', series_name);
%dir_name = 'G:/Imaging_files/20110427-2ph-cta_t48-Beads/Cta_t48_Spider-GFP-RedBeads(1)-converted/';
%% define parameters
%T1: the starting time point.
T1 = 1; ch = 1;
Bpass_high = 4; Bpass_low = 0;
thresh_frctn = 10; %1/10 of miximal
%% Enter properties
%note that X is verticle (row) while Y is horizontal (column)
%opposite from EDGE and common convention.
Xd = 128; %XY switched
Yd = 256; %XY switched
Zd = 40;
Td = 1428;
dx = 0.56; %dy in calibration
dy = 0.56; %dx in calibration
dz = 2.0;
dt = 8;

%% Pos_cell

Pos_cell  = cell(1, Td);
N_data = zeros(1, Td);

for t = Td:-1:T1,
    Stack_orig = zeros(Xd, Yd, Zd);
    
    for z=0:Zd-1, 
        % Create the appropriate file name
        
        if (Td >= 1001), %suitable for the two photon data NOT leica
            if (t<10),
                t_char = strcat('00', num2str(t));
            elseif (t>=10) && (t<100),
                t_char = strcat('0', num2str(t));
            elseif (t>=100) && (t<1000),
                t_char = num2str(t);
            else
                t_char = num2str(t);
            end
        elseif (Td < 1001) && (Td >=1), %%
            if (t<10),
                t_char = strcat('00', num2str(t));
            elseif (t>=10) && (t<100),
                t_char = strcat('0', num2str(t));
            else
                t_char = num2str(t);
            end
            %{
        elseif (Td < 101),
            if (t<10),
                t_char = strcat('0', num2str(t));
            else
                t_char = num2str(t);
            end
            %}
        end
        
        if (Zd >= 11),
            if (z<10),
                z_char = strcat('0', num2str(z));
            else 
                z_char = num2str(z);
            end
        elseif (Zd < 11),
            z_char = num2str(z);
        end
        
        if ch == 0,
            file_ch0 = strcat(dir_name, series_name, '_t', t_char, '_z', z_char, '_ch00.tif');
            im = imread(file_ch0);
        elseif ch == 1,
            file_ch1 = strcat(dir_name, series_name, '_t', t_char, '_z', z_char, '_ch01.tif');
            im = imread(file_ch1);
        end
        
        if size(im, 3) == 3,
            Stack_orig(:, :, z+1) = im(:, :, 1);
        else
            Stack_orig(:, :, z+1) = im;
        end
    end
    
    Stack_Bpass = Bpass3(Stack_orig, Bpass_low, Bpass_high);
    thresh_cut = max(max(max(Stack_Bpass)))/thresh_frctn;
    
    Pos_pixel_accuracy = pkfnd3_no_cut(Stack_Bpass, thresh_cut, 2);
    [AP] = gaussfit3_no_cut(Pos_pixel_accuracy, Stack_Bpass, 3, dx, dy, dz);
    
    if ~isempty(AP),
        X = AP(:, 1)*dx;
        Y = AP(:, 2)*dy;
        Z = AP(:, 3)*dz;
        N = length(X);
        Pos_cell{t} = [X, Y, Z];
    else
        Pos_cell{t} = [0, 0, 0];
    end
    
    %{
    close all;
    Pos_pix = [AP(:, 1), AP(:, 2), AP(:, 3)];
    Z_pf = Get_pixel_stack(Pos_pix, Stack_Bpass, 100, 2);

    for i=1:size(Z_pf, 3),
        h = figure;
        set(h, 'Position', [400, 100, 700, 900]);
        subplot(2,1,1), image(Stack_Bpass(:, :, i)), title('bypassed');
        subplot(2,1,2), image(Z_pf(:, :, i)), title('gauss fitted');
    end
    keyboard;
    %}
    
    disp(['Read out of time ', num2str(t), ' complete']);
    
end

%% Get trajectories   

T_cell = Triangular_Trace(Pos_cell);
T_end = length(T_cell);

Tri_bool = cell(1, T_end);
for i=1:T_end,
   Tri_bool{i} = zeros(size(T_cell{i}, 1), 1); 
end

Traj_paint = 0;
T_traj = cell(1);
for i=1:T_end,
   for j=1:size(T_cell{i},1), 
        if Tri_bool{i}(j) == 0,
           % If we havent began picking the trajectory here then begin
           Traj_paint = Traj_paint + 1; step = i; cur_id = j;
           % take first position
           Traj_tmp = [Pos_cell{step}(T_cell{step}(cur_id, 1), :), step];
           while (step < (T_end-1)),
               Tri_bool{step}(cur_id) = Traj_paint;
               Traj_tmp = [Traj_tmp;[Pos_cell{step+1}(T_cell{step}(cur_id, 2), :), step+1]];
               % Then check whether the next step exists
               if sum(T_cell{step}(cur_id, 2) == T_cell{step+1}(:, 1)) > 0,
                     
                     cur_id = find(T_cell{step}(cur_id, 2) == T_cell{step+1}(:, 1), 1);
                     step = step + 1;
                     if Tri_bool{step}(cur_id) > 0,
                         break;
                     end
       
               else
                   break;
               end
           end
           T_traj{Traj_paint} = Traj_tmp;
        end
   end
end
All_traj = T_traj;

disp('Aquiring of all Trajectories complete');

%% Save files

Prop.Xd = Xd;
Prop.dx = dx;
Prop.Yd = Yd;
Prop.dy = dy;
Prop.Zd = Zd;
Prop.dz = dz;
Prop.Td = Td;
Prop.dt = dt;
Prop.Series_name = strcat(folder_name, '_', series_name);

save_file = strcat(folder_name, '_', series_name, '.mat');
save(save_file, 'Pos_cell', 'All_traj', 'Prop');

end

