#include <iostream>
#include <fstream>
using namespace std;
#include <math.h>

#include "auxlrs.h"



//#include "b_par.h"
#include "F_par.h"



int main()
{
	
	//=====================================================================================================================================
	// Initialize variable
	int i=0, j=0, k=0, t=0, mt_id = 0;
	
	// The space for the potential
	
	#include "a_par.h"
	
	const double dx = Lx/NumX; // in micrometers
	const double dy = Ly/NumY;
	const double dz = Lz/NumZ;
	
	double* MT_x = new double[MT_num]; // position of the microtubili
	double* MT_y = new double[MT_num];
	
	// Position the microtubuli
	cout << "The microtubuli are positioned at [" << endl;
	for (i=0; i<(MT_num-1); i++) {
		MT_x[i] = (3.0*MT_init_pos)*cos(i*3.14159265/3.00);
		MT_y[i] = (3.0*MT_init_pos)*sin(i*3.14159265/3.00);
		cout << MT_x[i] << ", " << MT_y[i] << endl;
	}
	MT_x[6] = 0;
	MT_y[6] = 0;
	
	cout << "The microtubuli radius is " << MT_rad << endl;

	// Position of the particles
	double* X = new double[N];
	double* Y = new double[N];
	double* Z = new double[N];
	
	// Initialize the potential (NumX by NumY by NumZ double matrix)
	double *** V = new double**[NumX];
	
	for (i=0; i<NumX; i++) {
		V[i] = new double*[NumY];
		for (j=0; j<NumY; j++) {
			V[i][j] = new double[NumZ];
		}
	}
	
	// These are the variables to deal with the potential
	double X_prv, X_nxt, Y_prv, Y_nxt, Z_prv, Z_nxt;
	int X_prv_id, X_nxt_id, Y_prv_id, Y_nxt_id, Z_prv_id, Z_nxt_id;
	
	int X_id, Y_id, Z_id;
	
	double P_prv;
	
	
	
	long * idum = new long;
	*(idum) = 100;
	
	//=====================================================================================================================================
	// Assing initial positions for the particles
	cout << "Now Initializing Data and the Potential" << endl;
	for (i=0; i<N; i++) {
		X[i] = 0.5*NumX*dx;// + 2.0;
		Y[i] = 0.5*NumY*dy;
		Z[i] = 0.5*NumZ*dz;
	}
	

	
	
	//
	// Initialize the potential
	//*
	for (i=0; i<NumX; i++) {
		for (j=0; j<NumY; j++) {
			for (k=0; k<NumZ; k++) {
				V[i][j][k] = 0.0;
				
				if (i<side_cut || i>=(NumX - side_cut) || j<side_cut || j>=(NumY - side_cut) || k<side_cut || k>=(NumZ - side_cut)) {
					V[i][j][k] = 100.0;
				}
				else if (sqrt((i - NumX/2)*(i - NumX/2)*dx*dx + (j - NumY/2)*(j - NumY/2)*dy*dy) < rad_cut) {
					V[i][j][k] = 0.0;
				}
				else {
					V[i][j][k] = 100.0;
				}
				
				for (mt_id=0; mt_id<MT_num; mt_id++) {
					if (sqrt(pow((i-NumX/2)*dx - MT_x[mt_id], 2) + pow((j-NumY/2)*dx - MT_y[mt_id], 2)) < MT_rad) {
						V[i][j][k] = 100.0;
					}
				}

			}
		}
	}
	
	Record_Potential(V, NumX, NumY, NumZ, dx, dy, dz);
	//return 0;
	
	for (i=0; i<NumX; i++) {
		for (j=0; j<NumY; j++) {
			for (k=0; k<NumZ; k++) {
				V[i][j][k] = exp(-V[i][j][k]);
			}
		}
	}
	cout << "Initialization Complete!"<<endl;
	
	//=====================================================================================================================================
	
	
	//=====================================================================================================================================
	// Create the file
	cout << "Recording the data..."<<endl;
	Record_Init(dt, D_step, N, t_sample);
	
	cout << "Recording complete!"<<endl;
	//return 0;
	
	//=====================================================================================================================================
	// Time loop
	cout << "Now running time loop..."<<endl;
	for (t=0; t<=t_end; t++) {
	
		// Record the data
		if (fmod(t, t_sample) == 0){
			cout << "Iteration # " << t <<endl;
			if (t >=t_record){
				Record_data(X, Y, Z, N);
				cout << "Now recording";
			}
		}
		

		//==============================================================================================
		// Move the particles diffusivelly through the potential
		for (i=0; i<N; i++) {
			
			// The coordinate of the current box
			X_id = (int) (X[i]/dx);
			Y_id = (int) (Y[i]/dy);
			Z_id = (int) (Z[i]/dz);
			
			//*
			// Move X first
			X_prv = X[i] - D_step;
			X_nxt = X[i] + D_step;
			
			X_prv_id = (int) (X_prv/dx);
			X_nxt_id = (int) (X_nxt/dy);
			
			if (X_prv_id == X_nxt_id) {// If both states are in the same box along X
				(ran2(idum) > .5) ? X[i] = X_prv : X[i] = X_nxt; 
			}
			else { // If they are in different boxes
				P_prv = V[X_prv_id][Y_id][Z_id]/(V[X_prv_id][Y_id][Z_id] + V[X_nxt_id][Y_id][Z_id]);
				(ran2(idum) < P_prv) ? X[i] = X_prv : X[i] = X_nxt;
			}
			//*/
			//*
			// Move Y
			Y_prv = Y[i] - D_step;
			Y_nxt = Y[i] + D_step;
			
			Y_prv_id = (int) (Y_prv/dy);
			Y_nxt_id = (int) (Y_nxt/dy);
			
			if (Y_prv_id == Y_nxt_id) { // If both states are in the same box along Y
				(ran2(idum) > .5) ? Y[i] = Y_prv : Y[i] = Y_nxt;
			}
			else { // If they are in different boxes along Y
				
				P_prv = V[X_id][Y_prv_id][Z_id]/(V[X_id][Y_prv_id][Z_id] + V[X_id][Y_nxt_id][Z_id]);
				(ran2(idum) < P_prv) ? Y[i] = Y_prv : Y[i] = Y_nxt;
			}
			//*/
			//*
			// Move Z
			Z_prv = Z[i] - D_step;
			Z_nxt = Z[i] + D_step;
			
			Z_prv_id = (int) (Z_prv/dz);
			Z_nxt_id = (int) (Z_nxt/dz);
			
			if (Z_prv_id == Z_nxt_id) { // If both states are in the same box along Z
				(ran2(idum) > .5) ? Z[i] = Z_prv : Z[i] = Z_nxt;
			}
			else { // If they are in different boxes along Z
				P_prv = V[X_id][Y_id][Z_prv_id]/(V[X_id][Y_id][Z_prv_id] + V[X_id][Y_id][Z_nxt_id]);
				(ran2(idum) < P_prv) ? Z[i] = Z_prv : Z[i] = Z_nxt;
			}
			 //*/
		}
		
		
		
		
	}
	
	
	
	return 0;
	
}
